/* User Selector Applet: applet.c
 * 
 * Copyright (C) 2004 James M. Cape <jcape@ignore-your.tv>.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif /* HAVE_CONFIG_H */

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <pwd.h>
#include <string.h>

#include <X11/Xauth.h>

#include <gtk/gtkalignment.h>
#include <gtk/gtkcombobox.h>
#include <gtk/gtkexpander.h>
#include <gtk/gtkfilechooser.h>
#include <gtk/gtkframe.h>
#include <gtk/gtkhbox.h>
#include <gtk/gtkiconfactory.h>
#include <gtk/gtkicontheme.h>
#include <gtk/gtkimage.h>
#include <gtk/gtkimagemenuitem.h>
#include <gtk/gtklabel.h>
#include <gtk/gtkmenu.h>
#include <gtk/gtkmenubar.h>
#include <gtk/gtkmessagedialog.h>
#include <gtk/gtkradiobutton.h>
#include <gtk/gtkseparatormenuitem.h>
#include <gtk/gtkstock.h>
#include <gtk/gtkvbox.h>

#include <gconf/gconf-client.h>

#include <libgnome/gnome-exec.h>
#include <libgnome/gnome-config.h>

#include <libgnomeui/gnome-about.h>
#include <libgnomeui/gnome-pixmap-entry.h>

#include <panel-applet.h>
#include <panel-applet-gconf.h>

#include <esco-gdm.h>
#include <escoimagebutton.h>


/* ******** *
 *  Macros  *
 * ******** */

#define APPLET_DATA_QUARK	(applet_data_get_quark ())
#define MENUITEM_USER_KEY	(menuitem_user_key_get_quark ())
#define RADIO_SCREEN_KEY	(radiobutton_screen_key_get_quark ())


/* ***************** *
 *  Data Structures  *
 * ***************** */

typedef struct _AppletData AppletData;

struct _AppletData
{
  PanelApplet *applet;
  GConfClient *client;
  gchar *applet_dir;

  GtkWidget *my_menuitem_img;
  GtkWidget *my_menuitem_label;
  GtkWidget *separator_item;
  GtkWidget *new_screen_image;
  GtkWidget *new_screen_item;
  GtkWidget *new_window_image;
  GtkWidget *new_window_item;
  GtkWidget *about;

  GList *users;

  gint img_size;
};


/* *********************** *
 *  Applet Data Functions  *
 * *********************** */

static void
applet_data_free (AppletData * data)
{
  if (data != NULL)
    {
      gconf_client_remove_dir (data->client, data->applet_dir, NULL);
      gconf_client_remove_dir (data->client, "/apps/user-selector-applet", NULL);
      g_object_unref (data->client);
      g_free (data->applet_dir);
      g_list_foreach (data->users, (GFunc) g_object_unref, NULL);
      g_list_free (data->users);

      g_free (data);
    }
}


/* *************************** *
 *  Object Data Key Functions  *
 * *************************** */

static GQuark
applet_data_get_quark (void)
{
  static GQuark quark = 0;

  if (quark == 0)
    quark = g_quark_from_static_string ("applet-applet-data");

  return quark;
}


static GQuark
menuitem_user_key_get_quark (void)
{
  static GQuark quark = 0;

  if (quark == 0)
    quark = g_quark_from_static_string ("applet-menuitem-user");

  return quark;
}


static GQuark
radiobutton_screen_key_get_quark (void)
{
  static GQuark quark = 0;

  if (quark == 0)
    quark = g_quark_from_static_string ("applet-radiobutton-screen");

  return quark;
}


/* ******************* *
 *  Utility Functions  *
 * ******************* */

static const GSList *
get_all_mime_types (void)
{
  static GSList *mime_types = NULL;

  if (mime_types == NULL)
    {
      GSList *formats;

      for (formats = gdk_pixbuf_get_formats (); formats != NULL;
	   formats = g_slist_remove_link (formats, formats))
	{
	  guint i;
	  gchar **format_types;

	  format_types = gdk_pixbuf_format_get_mime_types (formats->data);
	  for (i = 0; format_types[i] != NULL; i++)
	    mime_types = g_slist_prepend (mime_types, format_types[i]);

	  g_free (format_types);
	}
    }

  return mime_types;
}


static void
lock_screen (void)
{
  gchar *argv[3] = {"xscreensaver-command", "-lock", NULL};

  if (gnome_execute_async (g_get_home_dir (), 2, argv) < 0)
    g_warning (_("Can't lock screen"));

  argv[1] = "-throttle";
  if (gnome_execute_async (g_get_home_dir (), 2, argv) < 0)
    g_warning (_("Can't disable xscreensaver display hacks"));
}


static void
show_error_dialog (const gchar * msgtitle, const gchar * msg, const gchar * detail)
{
  GtkWidget *dialog;
  GtkWidget *expander;

  if (detail != NULL)
    {
      GtkWidget *alignment;
      GtkWidget *label;

      expander = gtk_expander_new (_("Details"));
      gtk_widget_show (expander);

      alignment = gtk_alignment_new (0.0, 0.0, 0.0, 0.0);
      gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 6, 0, 12, 0);
      gtk_container_add (GTK_CONTAINER (expander), alignment);
      gtk_widget_show (alignment);

      label = gtk_label_new (detail);
      gtk_label_set_selectable (GTK_LABEL (label), TRUE);
      gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
      gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
      gtk_container_add (GTK_CONTAINER (alignment), label);
      gtk_widget_show (label);
    }
  else
    {
      expander = NULL;
    }

  dialog = ve_hig_dialog_new (NULL, GTK_DIALOG_NO_SEPARATOR, GTK_STOCK_DIALOG_ERROR, GTK_BUTTONS_OK,
			      expander, _("Error - User Selector"), FALSE, msgtitle, msg);
  gtk_dialog_run (GTK_DIALOG (dialog));
  gtk_widget_destroy (dialog);
}


static inline void
run_multi_vt_selector (EscoUser * user)
{
  GtkWidget *dialog, *alignment, *radio_box, *radio;
  GSList *group;
  gchar *text;
  gint response;
  const GList *screens;
  const gchar *name;
  EscoScreen *this_screen;

  this_screen = esco_screen_manager_get_this_screen (esco_screen_manager_get_default (), NULL);

  alignment = gtk_alignment_new (0.0, 0.0, 1.0, 0.0);
  gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 0, 0, 12, 0);
  gtk_widget_show (alignment);

  radio_box = gtk_vbox_new (TRUE, 3);
  gtk_container_add (GTK_CONTAINER (alignment), radio_box);
  gtk_widget_show (radio_box);

  group = NULL;
  name = esco_user_get_display_name (user);
  for (screens = esco_user_get_screens (user); screens != NULL; screens = screens->next)
    {
      if (this_screen != ESCO_SCREEN (screens->data))
	{
	  text = g_strdup_printf (_("%s on screen _%u"), name, ESCO_SCREEN (screens->data)->vt);
	  radio = gtk_radio_button_new_with_mnemonic (group, text);
	  g_free (text);
	  g_object_set_qdata (G_OBJECT (radio), RADIO_SCREEN_KEY, screens->data);
	  gtk_box_pack_start (GTK_BOX (radio_box), radio, FALSE, FALSE, 0);
	  gtk_widget_show (radio);

	  group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radio));
	}
    }
  esco_screen_unref (this_screen);

  dialog = ve_hig_dialog_new (NULL, GTK_DIALOG_NO_SEPARATOR, GTK_STOCK_DIALOG_QUESTION,
			      GTK_BUTTONS_NONE, alignment, _("Select Login - User Selector"),
			      TRUE, _("Multiple Logins Found"),
			      _("<i>%s</i> is logged in more than once. Which login do you want to "
				"go to?"), name);
  gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
  gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_JUMP_TO, GTK_RESPONSE_OK);

  response = gtk_dialog_run (GTK_DIALOG (dialog));

  if (response == GTK_RESPONSE_OK)
    {
      while (group != NULL)
	{
	  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (group->data)))
	    {
	      GError *error = NULL;

	      if (!esco_screen_manager_set_screen (esco_screen_manager_get_default (),
						   g_object_get_qdata (group->data,
								       RADIO_SCREEN_KEY), &error))
		{
		  show_error_dialog (_("Error Switching To User"),
				     _("There was an error switching to the user's screen."),
				     error->message);
		  g_error_free (error);
		}
	      else
		{
		  lock_screen ();
		}
	      break;
	    }

	  group = group->next;
	}
    }

  gtk_widget_destroy (dialog);
}


/* *********** *
 *  Callbacks  *
 * *********** */

/* New Screen Item */
static void
new_screen_activate_cb (GtkWidget * menuitem, gpointer user_data)
{
  GError *error = NULL;

  if (!esco_screen_manager_new_screen (esco_screen_manager_get_default (), &error))
    {
      show_error_dialog (_("Error Switching Users"),
			 _("An error occurred which prevented a new screen from being created."),
			 error->message);
      g_error_free (error);
    }
  else
    {
      lock_screen ();
    }
}


/* New Login Window Item */
static void
new_window_activate_cb (GtkWidget * menuitem, gpointer user_data)
{
  const gchar *auth_cookie;
  gchar *mit_cookie, *command, *result;

  auth_cookie = gdmcomm_get_auth_cookie ();

  if (auth_cookie == NULL)
    {
      gchar *msg = g_strdup_printf (_("The authentication file, %s, could not be read."),
				    XauFileName ());
      show_error_dialog (_("Error Starting Login Window"),
			 _("There was an error starting the login window."), msg);
      g_free (msg);
      return;
    }


  mit_cookie = gdmcomm_get_a_cookie (FALSE);
  command = g_strdup_printf ("%s %s %lu %s %s", GDM_SUP_FLEXI_XNEST, gdmcomm_get_display (),
			     (gulong) getuid (), mit_cookie, XauFileName ());
  g_free (mit_cookie);

  result = gdmcomm_call_gdm (command, auth_cookie, "2.3.90.4", 1);
  g_free (command);

  if (result == NULL || strncmp (result, "OK", 2) != 0)
    {
      show_error_dialog (_("Error Starting Login Window"),
			 _("There was an error starting the login window."),
			 gdmcomm_get_error_message (result, TRUE));
    }

  g_free (result);
}


/* **************** *
 *  User Callbacks  *
 * **************** */

static void
user_menuitem_activate_cb (GtkWidget * menuitem, EscoUser * user)
{
  const GList *screens;
  GError *error = NULL;

  screens = esco_user_get_screens (user);

  switch (g_list_length ((GList *) screens))
    {
      /* No screens */
    case 0:
      if (!esco_screen_manager_new_screen (esco_screen_manager_get_default (), &error))
	{
	  show_error_dialog (_("Error Switching Users"),
			     _("An error occurred which prevented a new screen from being "
			       "created."), error->message);
	  g_error_free (error);
	}
      else
	{
	  lock_screen ();
	}
      break;

      /* Just 1 screen */
    case 1:
      if (!esco_screen_manager_set_screen (esco_screen_manager_get_default (), screens->data,
					   &error))
	{
	  show_error_dialog (_("Error Switching Users"),
			     _("An error occurred which prevented switching to the user's screen."),
			     error->message);
	  g_error_free (error);
	}
      else
	{
	  lock_screen ();
	}
      break;

      /* 2 Screens (and our user), so we want to just switch to the other screen */
    case 2:
      if (esco_user_get_uid (user) == getuid ())
	{
	  EscoScreen *this_screen;

	  this_screen = esco_screen_manager_get_this_screen (esco_screen_manager_get_default (),
							     &error);
	  if (this_screen == NULL)
	    {
	      show_error_dialog (_("Error Switching Users"),
				 _("An error occurred which prevented retrieving the current "
				   "screen."), error->message);
	      g_error_free (error);
	    }
	  else
	    {
	      while (screens != NULL && this_screen == ESCO_SCREEN (screens->data))
	        screens = screens->next;
     
	      if (!esco_screen_manager_set_screen (esco_screen_manager_get_default (),
						   screens->data, &error))
		{
		  show_error_dialog (_("Error Switching Users"),
				     _("An error occurred which prevented switching to the user's "
				       "screen."), error->message);
		  g_error_free (error);
		}
	      else
		{
		  lock_screen ();
		}
	    }
  
	  esco_screen_unref (this_screen);

	  break;
	}
      /* More than 1 screen */
    default:
      run_multi_vt_selector (user);
      break;
    }
}


static void
user_notify_display_name_cb (EscoUser * user, GParamSpec * pspec, GtkBin * menuitem)
{
  gtk_label_set_text (GTK_LABEL (menuitem->child), esco_user_get_display_name (user));
}


static void
user_notify_icon_cb (EscoUser * user, GParamSpec * pspec, GtkImage * image)
{
  gtk_image_set_from_pixbuf (image, esco_user_get_icon (user));
}


static void
my_user_notify_screens_cb (EscoUser * user, GParamSpec * pspec, GtkWidget * menuitem)
{
  if (g_list_length ((GList *) esco_user_get_screens (user)) < 2)
    gtk_widget_set_sensitive (menuitem, FALSE);
  else
    gtk_widget_set_sensitive (menuitem, TRUE);
}


/* ************************ *
 *  Prefs Dialog Callbacks  *
 * ************************ */

static void
tiny_display_toggled_cb (GtkToggleButton *button, AppletData *data)
{
  gboolean active;

  active = gtk_toggle_button_get_active (button);

  panel_applet_gconf_set_bool (data->applet, "tiny-display", active, NULL);
}


static gboolean
photo_file_filter (const GtkFileFilterInfo *info, gpointer data)
{
  struct stat st;
  gssize max_user_size;
  const GSList *list;

  if (stat (info->filename, &st) < 0)
    return FALSE;

  if ((st.st_mode & (S_IWUSR | S_IWGRP | S_IWOTH)) == 0)
    return FALSE;

  max_user_size = esco_gdm_config_get_max_user_size ();

  if (st.st_size > max_user_size)
    return FALSE;

  if (info->contains & GTK_FILE_FILTER_MIME_TYPE)
    {
      for (list = get_all_mime_types (); list != NULL; list = list->next)
	{
	  if (strcmp (list->data, info->mime_type) == 0)
	    return TRUE;
	}

      return FALSE;
    }

  return TRUE;
}


static void
photo_selection_changed_cb (GtkFileChooser *chooser, gpointer data)
{
  gchar *filename;
  gint srcfd;

  filename = gtk_file_chooser_get_filename (chooser);

  srcfd = open (filename, O_RDONLY);
  if (srcfd < 0)
    {
      gchar *msg;

      msg = g_strdup_printf (_("The file `%s' could not be opened."), filename);
      show_error_dialog (_("Photo File Unreadable"), msg, g_strerror (errno));
      g_free (msg);
    }
  else
    {
      gchar *face_file;
      gint destfd;
    
      face_file = g_build_filename (g_get_home_dir (), ".face", NULL);
      destfd = open (face_file, (O_WRONLY | O_CREAT), 0644);
      g_free (face_file);

      if (destfd < 0)
	{
	  show_error_dialog (_("Face File Unreadable"),
			     _("There was a problem opening the face file."),
			     g_strerror (errno));
	}
      else
	{
	  gchar buffer[8192];
	  gssize len;

	  do
	    {
	      len = read (srcfd, buffer, sizeof (buffer));
	      len = write (destfd, buffer, len);
	    }
	  while (len > 0);

	  gnome_config_set_string ("/gdmphotosetup/last/picture", filename);
	  gnome_config_sync_file ("/gdmphotosetup");
	  close (destfd);
	}

      close (srcfd);
    }

  g_free (filename);
}


/* ****************** *
 *  Applet Callbacks  *
 * ****************** */

static void
prefs_cb (BonoboUIComponent * ui_container, AppletData *data, const gchar * cname)
{
  static GtkWidget *dialog = NULL;
  GtkWidget *main_box, *vbox, *frame, *alignment, *radio, *photo;
  GtkFileFilter *filter;
  gchar *tmp;

  if (dialog != NULL)
    {
      gtk_window_present (GTK_WINDOW (dialog));
      return;
    }

  dialog = gtk_dialog_new_with_buttons (_("Preferences - User Selector"), NULL,
					GTK_DIALOG_NO_SEPARATOR,
					GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, NULL);
  gtk_window_set_role (GTK_WINDOW (dialog), "UserAppletPreferences");
  gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
  gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 12);
  g_object_add_weak_pointer (G_OBJECT (dialog), (gpointer *) &dialog);
  g_signal_connect (dialog, "response", G_CALLBACK (gtk_widget_destroy), NULL);

  main_box = gtk_vbox_new (FALSE, 12);
  gtk_container_set_border_width (GTK_CONTAINER (main_box), 5);
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), main_box);
  gtk_widget_show (main_box);

  tmp = g_strconcat ("<b>", _("Display"), "</b>", NULL);
  frame = gtk_frame_new (tmp);
  g_free (tmp);
  gtk_label_set_use_markup (GTK_LABEL (gtk_frame_get_label_widget (GTK_FRAME (frame))), TRUE);
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
  gtk_box_pack_start (GTK_BOX (main_box), frame, FALSE, FALSE, 0);
  gtk_widget_show (frame);

  alignment = gtk_alignment_new (0.0, 0.0, 1.0, 0.0);
  gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 6, 0, 18, 0);
  gtk_container_add (GTK_CONTAINER (frame), alignment);
  gtk_widget_show (alignment);

  vbox = gtk_vbox_new (FALSE, 3);
  gtk_container_add (GTK_CONTAINER (alignment), vbox);
  gtk_widget_show (vbox);

  radio = gtk_radio_button_new_with_mnemonic (NULL, _("Show the current _user name"));
  gtk_box_pack_start (GTK_BOX (vbox), radio, FALSE, FALSE, 0);
  gtk_widget_show (radio);

  radio = gtk_radio_button_new_with_mnemonic_from_widget (GTK_RADIO_BUTTON (radio),
							  _("Show an _icon"));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio),
				panel_applet_gconf_get_bool (data->applet, "tiny-display", NULL));
  g_signal_connect (radio, "toggled", G_CALLBACK (tiny_display_toggled_cb), data);
  gtk_box_pack_start (GTK_BOX (vbox), radio, FALSE, FALSE, 0);
  gtk_widget_show (radio);

  tmp = g_strconcat ("<b>", _("Photo"), "</b>", NULL);
  frame = gtk_frame_new (tmp);
  g_free (tmp);
  gtk_label_set_use_markup (GTK_LABEL (gtk_frame_get_label_widget (GTK_FRAME (frame))), TRUE);
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
  gtk_container_add (GTK_CONTAINER (main_box), frame);
  gtk_widget_show (frame);

  alignment = gtk_alignment_new (0.0, 0.0, 1.0, 1.0);
  gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 6, 0, 18, 0);
  gtk_container_add (GTK_CONTAINER (frame), alignment);
  gtk_widget_show (alignment);

  esco_gdm_config_init ();
  photo = esco_image_button_new (_("Select A Photo - User Selector"));
  esco_image_button_set_minimum_size (ESCO_IMAGE_BUTTON (photo), 64);
  tmp = gnome_config_get_string ("/gdmphotosetup/last/picture");
  if (tmp != NULL)
    {
      gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (photo), tmp);
      g_free (tmp);
    }
  /* Drop the config from memory when the photo widget is destroyed. */
  g_object_weak_ref (G_OBJECT (photo), (GWeakNotify) esco_gdm_config_shutdown, NULL);
  g_signal_connect (photo, "selection-changed", G_CALLBACK (photo_selection_changed_cb), NULL);
  gtk_container_add (GTK_CONTAINER (alignment), photo);
  gtk_widget_show (photo);

  filter = gtk_file_filter_new ();
  gtk_file_filter_set_name (filter, _("Photo Files"));
  gtk_file_filter_add_custom (filter, (GTK_FILE_FILTER_FILENAME | GTK_FILE_FILTER_MIME_TYPE),
			      photo_file_filter, data, NULL);
  gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (photo), filter);
  gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (photo), filter);

  gtk_window_present (GTK_WINDOW (dialog));
}


static void
admin_cb (BonoboUIComponent * ui_container, gpointer user_data, const gchar * cname)
{
  char *argv[] = { "users-admin", NULL };

  gnome_execute_async (g_get_home_dir (), 1, argv);
}


static void
about_cb (BonoboUIComponent * ui_container, gpointer user_data, const gchar * cname)
{
  static const gchar *authors[] = { "James M. Cape <jcape@ignore-your.tv>", NULL };
  AppletData *data = user_data;
  GdkPixbuf *pixbuf;

  if (data->about != NULL)
    {
      gtk_window_present (GTK_WINDOW (data->about));
      return;
    }

  pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), "config-users", 96, 0, NULL);

  data->about = gnome_about_new ("User Selector", VERSION,
				 "Copyright 2004 James M. Cape",
				 _("A menu to switch users for GNOME 2.6"),
				 authors, NULL, NULL, pixbuf);
  g_object_add_weak_pointer (G_OBJECT (data->about), (gpointer *) & data->about);
  gtk_window_present (GTK_WINDOW (data->about));
}


static gboolean
menubar_button_press_event_cb (GtkWidget * menubar, GdkEventButton * event)
{
  if (event->button != 1)
    g_signal_stop_emission_by_name (menubar, "button-press-event");

  esco_screen_manager_update (esco_screen_manager_get_default (), NULL);

  return FALSE;
}


static void
theme_changed_cb (GtkIconTheme * theme, AppletData * data)
{
  GList *icon_list;
  gint width, height;
  GdkPixbuf *pixbuf;
  guint i;
  gint sizes[] = { 16, 24, 32, 48 };

  /* Icon Size */
  if (!gtk_icon_size_lookup (gtk_icon_size_from_name ("panel-menu"), &width, &height))
    data->img_size = 32;
  else
    data->img_size = MAX (MAX (width, height), 16);

  /* Default Icon List */
  icon_list = NULL;

  for (i = 0; i < G_N_ELEMENTS (sizes); i++)
    {
      pixbuf = gtk_icon_theme_load_icon (theme, "stock_people", sizes[i], 0, NULL);

      if (pixbuf != NULL)
	icon_list = g_list_prepend (icon_list, pixbuf);
    }

  gtk_window_set_default_icon_list (icon_list);

  g_list_foreach (icon_list, (GFunc) g_object_unref, NULL);
  g_list_free (icon_list);

  if (data->about != NULL)
    {
      pixbuf = gtk_icon_theme_load_icon (theme, "config-users", 96, 0, NULL);

      g_object_set (data->about, "pixbuf", pixbuf, NULL);
      g_object_unref (pixbuf);
    }

  /* X Image */
  pixbuf = gtk_icon_theme_load_icon (theme, "gdm", data->img_size, 0, NULL);
  if (pixbuf != NULL)
    {
      gtk_image_set_from_pixbuf (GTK_IMAGE (data->new_screen_image), pixbuf);
      g_object_unref (pixbuf);
    }

  /* Xnest Image */
  pixbuf = gtk_icon_theme_load_icon (theme, "gdm-xnest", data->img_size, 0, NULL);
  if (pixbuf != NULL)
    {
      gtk_image_set_from_pixbuf (GTK_IMAGE (data->new_window_image), pixbuf);
      g_object_unref (pixbuf);
    }
}


/* ***************** *
 *  GConf Functions  *
 * ***************** */

static void
client_notify (GConfClient *client, guint id, GConfEntry *entry, AppletData *data)
{
  const gchar *key;

  key = strrchr (entry->key, '/');

  if (key == NULL)
    return;

  key++;

  if (strcmp (key, "tiny-display") == 0)
    {
      if (gconf_value_get_bool (entry->value))
	{
	  gtk_widget_show (data->my_menuitem_img);
	  gtk_widget_hide (data->my_menuitem_label);
	}
      else
	{
	  gtk_widget_show (data->my_menuitem_label);
	  gtk_widget_hide (data->my_menuitem_img);
	}
    }
  else if (strcmp (key, "show-login-screen") == 0)
    {
      if (gconf_value_get_bool (entry->value))
	{
	  gtk_widget_show (data->new_screen_item);
	  gtk_widget_show (data->separator_item);
	}
      else
	{
	  gtk_widget_hide (data->new_screen_item);

	  if (!gconf_client_get_bool (client, "/apps/user-selector-applet/show-login-window", NULL))
	    gtk_widget_hide (data->separator_item);
	}
    }
  else if (strcmp (key, "show-login-window") == 0)
    {
      if (gconf_value_get_bool (entry->value))
	{
	  gtk_widget_show (data->new_window_item);
	  gtk_widget_show (data->separator_item);
	}
      else
	{
	  gtk_widget_hide (data->new_window_item);

	  if (!gconf_client_get_bool (client, "/apps/user-selector-applet/show-login-screen", NULL))
	    gtk_widget_hide (data->separator_item);
	}
    }
}

/* **************** *
 *  Init Functions  *
 * **************** */

static gboolean
applet_fill (PanelApplet * applet, const gchar * iid, gpointer user_data)
{
  const gchar *const menu_xml =
    "<popup name=\"button3\">\n"
    "  <menuitem name=\"Prefs Item\" verb=\"Prefs\""
    "    _label=\"Preferences...\"\n"
    "    pixtype=\"stock\" pixname=\"gtk-properties\"/>\n"
    "  <menuitem name=\"Users and Groups Item\" verb=\"Admin\""
    "    _label=\"Users and Groups...\"\n"
    "    pixtype=\"stock\" pixname=\"gtk-preferences\"/>\n"
    "  <menuitem name=\"About Item\" verb=\"About\" _label=\"About...\"\n"
    "    pixtype=\"stock\" pixname=\"gnome-stock-about\"/>\n" "</popup>\n";
  const BonoboUIVerb menu_verbs[] = {
    BONOBO_UI_VERB ("Prefs", (BonoboUIVerbFn) prefs_cb),
    BONOBO_UI_VERB ("Admin", admin_cb),
    BONOBO_UI_VERB ("About", about_cb),
    BONOBO_UI_VERB_END
  };
  AppletData *data;
  guint i;
  GtkWidget *menubar, *my_menuitem, *hbox, *menu, *menuitem;
  GList *icon_list;
  gint width, height;
  GtkIconTheme *theme;
  GdkPixbuf *pixbuf;
  struct passwd *pwent;
  gboolean show_screen, show_window;
  gint sizes[] = { 16, 24, 32, 48 };

  if (strcmp (iid, "OAFIID:GNOME_UserSelectorApplet") != 0)
    return FALSE;

  g_set_application_name (_("User Selector"));

  data = g_new0 (AppletData, 1);

  /* Icon Size */
  if (!gtk_icon_size_lookup (gtk_icon_size_from_name ("panel-menu"), &width, &height))
    data->img_size = 32;
  else
    data->img_size = MAX (MAX (width, height), 16);

  theme = gtk_icon_theme_get_default ();
  g_signal_connect (theme, "changed", G_CALLBACK (theme_changed_cb), data);

  /* Default Icon List */
  icon_list = NULL;
  for (i = 0; i < G_N_ELEMENTS (sizes); i++)
    {
      pixbuf = gtk_icon_theme_load_icon (theme, "stock_people", sizes[i], 0, NULL);

      if (pixbuf != NULL)
	icon_list = g_list_append (icon_list, pixbuf);
    }

  gtk_window_set_default_icon_list (icon_list);

  g_list_foreach (icon_list, (GFunc) g_object_unref, NULL);
  g_list_free (icon_list);

  /* Basic Widgets */
  data->applet = applet;
  gtk_container_set_border_width (GTK_CONTAINER (applet), 0);
  panel_applet_set_flags (applet, PANEL_APPLET_EXPAND_MINOR);
  panel_applet_add_preferences (applet, "/schemas/apps/user-selector-applet", NULL);

  menubar = gtk_menu_bar_new ();
  GTK_WIDGET_SET_FLAGS (menubar, GTK_WIDGET_FLAGS (menubar) | GTK_CAN_FOCUS);
  g_signal_connect (menubar, "button-press-event",
		    G_CALLBACK (menubar_button_press_event_cb), NULL);
  gtk_container_add (GTK_CONTAINER (applet), menubar);
  gtk_container_set_border_width (GTK_CONTAINER (menubar), 0);
  gtk_widget_show (menubar);

  my_menuitem = gtk_menu_item_new ();
  gtk_container_add (GTK_CONTAINER (menubar), my_menuitem);
  gtk_widget_show (my_menuitem);

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (my_menuitem), hbox);
  gtk_widget_show (hbox);

  pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), "stock_people",
				     panel_applet_get_size (applet) - 2 -
				     (GTK_WIDGET (applet)->style->ythickness * 2), 0, NULL);
  data->my_menuitem_img = gtk_image_new_from_pixbuf (pixbuf);
  g_object_unref (pixbuf);
  gtk_container_add (GTK_CONTAINER (hbox), data->my_menuitem_img);

  data->my_menuitem_label = gtk_label_new (NULL);
  gtk_container_add (GTK_CONTAINER (hbox), data->my_menuitem_label);

  if (panel_applet_gconf_get_bool (applet, "tiny-display", NULL))
    gtk_widget_show (data->my_menuitem_img);
  else
    gtk_widget_show (data->my_menuitem_label);

  menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (my_menuitem), menu);
  gtk_widget_show (menu);

  esco_screen_manager_update (esco_screen_manager_get_default (), NULL);

  setpwent ();
  esco_gdm_config_init ();

  pwent = getpwent ();

  while (pwent != NULL)
    {
      if (esco_user_is_visible (pwent))
	{
	  GtkWidget *image;
	  EscoUser *user;

	  user = esco_user_new_from_passwd (pwent, data->img_size);
	  data->users = g_list_prepend (data->users, user);

	  menuitem = gtk_image_menu_item_new_with_label (esco_user_get_display_name (user));
	  g_signal_connect (menuitem, "activate", G_CALLBACK (user_menuitem_activate_cb), user);
	  g_object_set_qdata (G_OBJECT (user), MENUITEM_USER_KEY, menuitem);

	  pixbuf = esco_user_get_icon (user);

	  if (pixbuf != NULL)
	    {
	      image = gtk_image_new_from_pixbuf (pixbuf);
	      g_object_unref (pixbuf);
	    }
	  else
	    {
	      image = gtk_image_new ();
	    }
	  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menuitem), image);
	  gtk_widget_show (image);

	  if (pwent->pw_uid == getuid ())
	    {
	      gtk_label_set_text (GTK_LABEL (data->my_menuitem_label),
				  esco_user_get_display_name (user));
	      g_signal_connect (user, "notify::screens", G_CALLBACK (my_user_notify_screens_cb),
			        menuitem);

	      if (g_list_length ((GList *) esco_user_get_screens (user)) < 2)
		{
		  gtk_widget_set_sensitive (GTK_WIDGET (menuitem), FALSE);
		}
	    }

	  g_signal_connect (user, "notify::display-name", G_CALLBACK (user_notify_display_name_cb),
			    menuitem);
	  g_signal_connect (user, "notify::icon", G_CALLBACK (user_notify_icon_cb), image);

	  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
	  gtk_widget_show (menuitem);
	}

      pwent = getpwent ();
    }

  esco_gdm_config_shutdown ();
  endpwent ();

  data->client = gconf_client_get_default ();
  gconf_client_add_dir (data->client, "/apps/user-selector-applet", GCONF_CLIENT_PRELOAD_ONELEVEL,
			NULL);
  data->applet_dir = panel_applet_gconf_get_full_key (applet, "blah");
  *(strrchr (data->applet_dir, '/')) = '\0';
  gconf_client_add_dir (data->client, data->applet_dir, GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
  show_screen = gconf_client_get_bool (data->client, "/apps/user-selector-applet/show-login-screen",
				       NULL);
  show_window = gconf_client_get_bool (data->client, "/apps/user-selector-applet/show-login-window",
				       NULL);
  gconf_client_notify_add (data->client, "/apps/user-selector-applet",
			   (GConfClientNotifyFunc) client_notify, data, NULL, NULL);
  gconf_client_notify_add (data->client, data->applet_dir, (GConfClientNotifyFunc) client_notify,
			   data, NULL, NULL);

  /* Separator Item */
  data->separator_item = gtk_separator_menu_item_new ();
  gtk_menu_shell_append (GTK_MENU_SHELL (menu), data->separator_item);

  if (show_screen || show_window)
    gtk_widget_show (data->separator_item);

  /* Login Screen Menuitem */
  data->new_screen_item = gtk_image_menu_item_new_with_label (_("Login Screen..."));
  gtk_menu_shell_append (GTK_MENU_SHELL (menu), data->new_screen_item);
  g_signal_connect (data->new_screen_item, "activate", G_CALLBACK (new_screen_activate_cb), NULL);
  if (show_screen)
    gtk_widget_show (data->new_screen_item);

  pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), "gdm", data->img_size, 0, NULL);

  data->new_screen_image = gtk_image_new ();
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (data->new_screen_item),
				 data->new_screen_image);
  gtk_widget_show (data->new_screen_image);

  pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), "gdm", data->img_size, 0, NULL);

  if (pixbuf != NULL)
    {
      gtk_image_set_from_pixbuf (GTK_IMAGE (data->new_screen_image), pixbuf);
      g_object_unref (pixbuf);
    }

  /* Login Window Menuitem */
  data->new_window_item = gtk_image_menu_item_new_with_label (_("Login Window..."));
  gtk_menu_shell_append (GTK_MENU_SHELL (menu), data->new_window_item);
  g_signal_connect (data->new_window_item, "activate", G_CALLBACK (new_window_activate_cb), NULL);
  if (show_window)
    gtk_widget_show (data->new_window_item);

  data->new_window_image = gtk_image_new ();
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (data->new_window_item),
				 data->new_window_image);
  gtk_widget_show (data->new_window_image);

  pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), "gdm-xnest", data->img_size, 0,
				     NULL);

  if (pixbuf != NULL)
    {
      gtk_image_set_from_pixbuf (GTK_IMAGE (data->new_window_image), pixbuf);
      g_object_unref (pixbuf);
    }

  panel_applet_setup_menu (applet, menu_xml, menu_verbs, data);

  gtk_widget_show (GTK_WIDGET (applet));

  gtk_rc_parse_string ("style \"panel-menubar-style\"\n"
		       "{\n"
		       "  GtkMenuBar::shadow-type = none\n"
		       "  GtkMenuBar::internal-padding = 0\n"
		       "}\n" "class \"GtkMenuBar\" style \"panel-menubar-style\"");

  g_object_set_qdata_full (G_OBJECT (applet), APPLET_DATA_QUARK, data,
			   (GDestroyNotify) applet_data_free);

  return TRUE;
}


/* ************************************************************************** *
 *  Main                                                                      *
 * ************************************************************************** */

PANEL_APPLET_BONOBO_FACTORY ("OAFIID:GNOME_UserSelectorApplet_Factory", PANEL_TYPE_APPLET,
			     "User Selector Applet", "0", applet_fill, NULL);
