/* Esco-GDM: esco-screen-manager.h
 * 
 * Copyright (C) 2004 James M. Cape <jcape@ignore-your.tv>.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __ESCO_SCREEN_MANAGER_H__
#define __ESCO_SCREEN_MANAGER_H__

#include <glib-object.h>

G_BEGIN_DECLS


#define ESCO_TYPE_SCREEN_MANAGER \
  (esco_screen_manager_get_type ())
#define ESCO_SCREEN_MANAGER(object) \
  (G_TYPE_CHECK_INSTANCE_CAST ((object), ESCO_TYPE_SCREEN_MANAGER, EscoScreenManager))
#define ESCO_SCREEN_MANAGER_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST ((klass), ESCO_TYPE_SCREEN_MANAGER, EscoScreenManagerClass))
#define ESCO_IS_SCREEN_MANAGER(object) \
  (G_TYPE_CHECK_INSTANCE_TYPE ((object), ESCO_TYPE_SCREEN_MANAGER))
#define ESCO_IS_SCREEN_MANAGER_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_TYPE ((klass), ESCO_TYPE_SCREEN_MANAGER))
#define ESCO_SCREEN_MANAGER_GET_CLASS(object) \
  (G_TYPE_INSTANCE_GET_CLASS ((object), ESCO_TYPE_SCREEN_MANAGER, EscoScreenManagerClass))
  
#define ESCO_TYPE_SCREEN \
  (esco_screen_get_type ())
#define ESCO_SCREEN(ptr) \
  ((EscoScreen *) (ptr))
#define ESCO_IS_SCREEN(ptr) \
  ((ptr) != NULL && G_TYPE_FROM_CLASS (ptr) == ESCO_TYPE_SCREEN)

#define ESCO_GDM_ERROR \
  (esco_gdm_error_get_quark ())


typedef struct _EscoScreenManager EscoScreenManager;
typedef struct _EscoScreenManagerClass EscoScreenManagerClass;

typedef struct _EscoScreen EscoScreen;
  

typedef enum
{
  ESCO_GDM_ERROR_GDM_NOT_RUNNING = -1,
  ESCO_GDM_ERROR_BAD_RESULT = -2,
  ESCO_GDM_ERROR_OLD_VERSION = 0,
  ESCO_GDM_ERROR_TOO_MANY_FLEXI = 1,
  ESCO_GDM_ERROR_X_ERROR = 2,
  ESCO_GDM_ERROR_X_FAILED = 3,
  ESCO_GDM_ERROR_TOO_MANY_SESSIONS = 4,
  ESCO_GDM_ERROR_XNEST_NOT_AVAILABLE = 5,
  ESCO_GDM_ERROR_X_NOT_AVAILABLE = 6,
  ESCO_GDM_ERROR_UNKNOWN_LOGOUT_ACTION = 7,
  ESCO_GDM_ERROR_VT_NOT_SUPPORTED = 8,
  ESCO_GDM_ERROR_INVALID_VT = 9,
  ESCO_GDM_ERROR_UNSUPPORTED_CONFIG_KEY = 50,
  ESCO_GDM_ERROR_AUTH_NOT_AVAILABLE = 100,
  ESCO_GDM_ERROR_TOO_MANY_MESSAGES = 200,
  ESCO_GDM_ERROR_UNKNOWN = 999
}
EscoGdmError;


GType esco_screen_manager_get_type (void) G_GNUC_CONST;

EscoScreenManager *esco_screen_manager_get_default (void);
EscoScreenManager *esco_screen_manager_new (void);

gboolean esco_screen_manager_update (EscoScreenManager *mgr, GError **err);

GList *esco_screen_manager_list_screens (EscoScreenManager *mgr);
GList *esco_screen_manager_list_screens_for_user (EscoScreenManager *mgr, const gchar *username);

EscoScreen *esco_screen_manager_get_this_screen (EscoScreenManager *mgr, GError **err);
EscoScreen *esco_screen_manager_get_screen (EscoScreenManager *mgr, guint vt);
gboolean esco_screen_manager_set_screen (EscoScreenManager *mgr, EscoScreen *screen, GError **err);
gboolean esco_screen_manager_new_screen (EscoScreenManager *mgr, GError **err);


struct _EscoScreen
{
  GTypeClass g_class;
  guint ref;

  guint vt;
  gchar *display;
  gchar *username;
};

GType esco_screen_get_type (void) G_GNUC_CONST;

EscoScreen *esco_screen_ref (EscoScreen *screen);
void esco_screen_unref (EscoScreen *screen);


GQuark esco_gdm_error_get_quark (void) G_GNUC_CONST;


G_END_DECLS

#endif /* __ESCO_SCREEN_MANAGER_H__ */
