
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include <string.h>

#include <gtk/gtk.h>

#include "escofilechooserentry.h"
#include "escoimagebutton.h"
#include "prop-editor.h"


/*
 * I say fix the GtkDialog defaults to be HIG-compliant and tell the
 * applications to fix their usage. Certainly this should be done in the 3.0
 * series.
 */
static void
win_style_set_cb (GtkWidget *win)
{
  gtk_container_set_border_width (GTK_CONTAINER (GTK_DIALOG (win)->vbox), 12);
  gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (win)->vbox), 24);
  gtk_container_set_border_width (GTK_CONTAINER (GTK_DIALOG (win)->action_area), 0);
  gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (win)->action_area), 6);
}


static gboolean
editor_delete_event_cb (GtkWidget *editor, gint response, gpointer user_data)
{
  gtk_widget_hide (editor);

  return TRUE;
}


static void
properties_button_clicked_cb (GtkWidget *button, GObject *entry)
{
  GtkWidget *editor;

  editor = g_object_get_data (entry, "properties-dialog");

  if (editor == NULL)
    {
      editor = create_prop_editor (G_OBJECT (entry), G_TYPE_INVALID);
      gtk_window_set_transient_for (GTK_WINDOW (editor),
				    GTK_WINDOW (gtk_widget_get_toplevel (button)));
      g_signal_connect (editor, "delete-event", G_CALLBACK (editor_delete_event_cb), NULL);
      g_object_set_data (entry, "properties-dialog", editor);
    }

  gtk_window_present (GTK_WINDOW (editor));
}


static const GSList *
get_all_mime_types (void)
{
  static GSList *mime_types = NULL;

  if (mime_types == NULL)
    {
      GSList *formats;

      for (formats = gdk_pixbuf_get_formats (); formats != NULL;
	   formats = g_slist_remove_link (formats, formats))
	{
	  guint i;
	  gchar **format_types;

	  format_types = gdk_pixbuf_format_get_mime_types (formats->data);
	  for (i = 0; format_types[i] != NULL; i++)
	    mime_types = g_slist_prepend (mime_types, format_types[i]);

	  g_free (format_types);
	}
    }

  return mime_types;
}


static gboolean
photo_file_filter (const GtkFileFilterInfo *info, gpointer data)
{
  struct stat st;
  gssize max_user_size = 64000;
  const GSList *list;

  if (stat (info->filename, &st) < 0)
    return FALSE;

  if (st.st_size > max_user_size)
    return FALSE;

  if (info->contains & GTK_FILE_FILTER_MIME_TYPE)
    {
      for (list = get_all_mime_types (); list != NULL; list = list->next)
	{
	  if (strcmp (list->data, info->mime_type) == 0)
	    return TRUE;
	}

      return FALSE;
    }

  return TRUE;
}


int
main (int argc, char *argv[])
{
  GtkWidget *win, *vbox, *frame, *alignment, *hbox, *label, *entry, *button;
  GtkFileFilter *filter;

  gtk_init (&argc, &argv);

  win = gtk_dialog_new_with_buttons ("Escobar Widgets", NULL, GTK_DIALOG_NO_SEPARATOR,
				     GTK_STOCK_QUIT, GTK_RESPONSE_CLOSE, NULL);
  g_signal_connect (win, "style-set", G_CALLBACK (win_style_set_cb), NULL);
  g_signal_connect (win, "response", G_CALLBACK (gtk_main_quit), NULL);

  vbox = gtk_vbox_new (FALSE, 18);
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (win)->vbox), vbox);
  gtk_widget_show (vbox);

  frame = gtk_frame_new ("<b>EscoFileChooserEntry</b>");
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
  gtk_label_set_use_markup (GTK_LABEL (gtk_frame_get_label_widget (GTK_FRAME (frame))), TRUE);
  gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
  gtk_widget_show (frame);

  alignment = gtk_alignment_new (0.0, 0.0, 1.0, 1.0);
  gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 6, 0, 12, 0);
  gtk_container_add (GTK_CONTAINER (frame), alignment);
  gtk_widget_show (alignment);

  hbox = gtk_hbox_new (FALSE, 12);
  gtk_container_add (GTK_CONTAINER (alignment), hbox);
  gtk_widget_show (hbox);

  label = gtk_label_new ("Open:");
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
  gtk_widget_show (label);

  entry = esco_file_chooser_entry_new ("Select A File - testescoui");
  gtk_container_add (GTK_CONTAINER (hbox), entry);
  gtk_widget_show (entry);

  button = gtk_button_new_with_label ("Properties...");
  g_signal_connect (button, "clicked", G_CALLBACK (properties_button_clicked_cb), entry);
  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
  gtk_widget_show (button);

  frame = gtk_frame_new ("<b>EscoImageButton</b>");
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
  gtk_label_set_use_markup (GTK_LABEL (gtk_frame_get_label_widget (GTK_FRAME (frame))), TRUE);
  gtk_container_add (GTK_CONTAINER (vbox), frame);
  gtk_widget_show (frame);

  alignment = gtk_alignment_new (0.0, 0.0, 1.0, 1.0);
  gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 6, 0, 12, 0);
  gtk_container_add (GTK_CONTAINER (frame), alignment);
  gtk_widget_show (alignment);

  hbox = gtk_hbox_new (FALSE, 12);
  gtk_container_add (GTK_CONTAINER (alignment), hbox);
  gtk_widget_show (hbox);

  entry = esco_image_button_new ("Select an Image - testescoui");
  gtk_container_add (GTK_CONTAINER (hbox), entry);
  gtk_widget_show (entry);
  
  filter = gtk_file_filter_new ();
  gtk_file_filter_set_name (filter, "Images < 64k");
  gtk_file_filter_add_custom (filter, (GTK_FILE_FILTER_MIME_TYPE | GTK_FILE_FILTER_FILENAME),
			      photo_file_filter, NULL, NULL);
  gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (entry), filter);
  gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (entry), filter);

  button = gtk_button_new_with_label ("Properties...");
  g_signal_connect (button, "clicked", G_CALLBACK (properties_button_clicked_cb), entry);
  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
  gtk_widget_show (button);

  gtk_window_present (GTK_WINDOW (win));

  gtk_main ();

  gtk_widget_destroy (win);

  return 0;
}
